#include <vector>
#include <string>

using namespace std;

#include <gtest/gtest.h>

#include <cxcparamx/simple.h>
using namespace cxcparamx;

#include "argv.h"

constexpr double dvalue( 3.14159 );
constexpr double fvalue( 3.14159f );
constexpr int    ivalue( 4321 );
const string svalue( "humpty dumpty" );

using std::vector;
using std::string;

class Args : public ::testing::Test {

protected:

    static void SetUpTestCase() {

	vector<string> args = { "test", "string=" + svalue, "real=3.14159",  "integer=4321", "true=no", "false=yes" };

	Argv argv( args );

	pf = new simple( argv.size(), argv(), "r" );
    }

    static void TearDownTestCase() {
	if ( pf )
	    delete pf;
    }

    static cxcparamx::simple* pf;

};

cxcparamx::simple* Args::pf = NULL;


TEST_F(Args, String) {

    string value;


    ASSERT_NO_THROW( pf->get( "string", value ) );

    EXPECT_EQ( svalue, value );
}

TEST_F(Args, Double) {

    double value;

    ASSERT_NO_THROW( pf->get( "real", value ) );

    EXPECT_DOUBLE_EQ( dvalue, value );
}

TEST_F(Args, Float) {

    float value;

    ASSERT_NO_THROW( pf->get( "real", value ) );

    EXPECT_FLOAT_EQ( fvalue, value );
}

TEST_F(Args, Short) {

    short value;

    ASSERT_NO_THROW( pf->get( "integer", value ) );

    EXPECT_EQ( ivalue, value );
}

TEST_F(Args, Integer) {

    int value;

    ASSERT_NO_THROW( pf->get( "integer", value ) );

    EXPECT_EQ( ivalue, value );
}

TEST_F(Args, Long) {

    long value;

    ASSERT_NO_THROW( pf->get( "integer", value ) );

    EXPECT_EQ( ivalue, value );
}

TEST_F(Args, BoolTrue) {

    bool value;

    ASSERT_NO_THROW( pf->get( "true", value ) );

    EXPECT_FALSE( value );
}

TEST_F(Args, BoolFalse) {

    bool value;

    ASSERT_NO_THROW( pf->get( "false", value ) );

    EXPECT_TRUE( value );
}
